/*******************************************************************************
 * Copyright (c) 2010, 2017 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     Lars Vogel <Lars.Vogel@gmail.com> - Bug 420836
 *     Patrik Suzzi <psuzzi@gmail.com> - Bug 497591, 501250, 512385
 *     Ingo Mohr <tellastory73@gmail.com> - Bug 566842
 *******************************************************************************/

@import url("platform:/plugin/org.eclipse.ui.themes/css/e4_basestyle.css");

ColorDefinition#org-eclipse-ui-workbench-ACTIVE_UNSELECTED_TABS_COLOR_START {
	color: rgb(255, 255, 255);
}

ColorDefinition#org-eclipse-ui-workbench-ACTIVE_UNSELECTED_TABS_COLOR_END {
	color: rgb(255, 255, 255);
}

ColorDefinition#org-eclipse-ui-workbench-INACTIVE_UNSELECTED_TABS_COLOR_START {
	color: rgb(255, 255, 255);
}

ColorDefinition#org-eclipse-ui-workbench-INACTIVE_UNSELECTED_TABS_COLOR_END {
	color: rgb(255, 255, 255);
}

ColorDefinition#org-eclipse-ui-workbench-ACTIVE_TAB_OUTER_KEYLINE_COLOR {
	color: rgb(230, 230, 230);
}

ColorDefinition#org-eclipse-ui-workbench-ACTIVE_TAB_BG_START {
	color: rgb(230, 230, 230);
}

ColorDefinition#org-eclipse-ui-workbench-ACTIVE_TAB_BG_END {
	color: rgb(255, 255, 255);
}

ColorDefinition#org-eclipse-ui-workbench-INACTIVE_TAB_BG_START {
	color: rgb(240, 240, 240);
}

ColorDefinition#org-eclipse-ui-workbench-INACTIVE_TAB_BG_END {
	color: rgb(255, 255, 255);
}

.MTrimmedWindow {
	background-color: rgb(255, 255, 255);
}

.MTrimBar {
	background-color: rgb(255, 255, 255);
}

.MTrimBar#org-eclipse-ui-main-toolbar {
	background-color: rgb(255, 255, 255);
}

.MToolControl.TrimStack {
	frame-image: url(./macTSFrame.png);
	handle-image: url(./macHandle.png);
	frame-image-rotated: url(./macTSFrame-rotated.png);
	handle-image-rotated: url(./macHandle-rotated.png);
	frame-cuts: 5px 1px 5px 16px;
}

.MToolBar.Draggable {
	handle-image: url(./macHandle.png);
}

.MPartStack {
	swt-simple: false;
	swt-unselected-tabs-color: rgb(255, 255, 255);
	swt-outer-keyline-color: rgb(230, 230, 230);
	swt-inner-keyline-color: rgb(200, 200, 200);
	swt-tab-outline: rgb(230, 230, 230);
}

.MPartStack.active > Composite {
	swt-selected-tab-fill: '#org-eclipse-ui-workbench-ACTIVE_TAB_BG_START' '#org-eclipse-ui-workbench-ACTIVE_TAB_BG_END' 100% 5%;
	background-color:  rgb(255, 255, 255);
}

.MPartStack.active {
	swt-tab-outline: rgb(180, 180, 180);
	swt-shadow-visible: false;
	swt-outer-keyline-color: rgb(180, 180, 180);
}

CTabFolder.MArea {
	background-color: rgb(255, 255, 255);
	swt-selected-tab-fill: rgb(255, 255, 255);
	swt-unselected-tabs-color: rgb(255, 255, 255);
	swt-outer-keyline-color: rgb(230, 230, 230);
	swt-inner-keyline-color: rgb(230, 230, 230);
	swt-tab-outline: rgb(230, 230, 230);
}

CTabFolder {
  swt-selected-tabs-background: rgb(255, 255, 255);
}

CTabFolder CTabItem:selected {
  background-color: rgb(230, 230, 230);
}

CTabFolder Canvas {
  background-color: rgb(255, 255, 255);
}
