/*******************************************************************************
 * Copyright (c) 2014 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/

ThemesExtension {
	color-definition: '#org-eclipse-e4-demo-cssbridge-ui-views-theme-shell-background';
}

/* for IDs see the class: org.eclipse.e4.demo.cssbridge.ui.views.Theme.FoldersView */
FontDefinition#org-eclipse-e4-demo-cssbridge-ui-views-theme-foldersview-mailbox-name-font {
	font-family: Times;
	font-size: 10px;
}

FontDefinition#org-eclipse-e4-demo-cssbridge-ui-views-theme-foldersview-folder-type-font {
	font-family: Tahoma;
	font-size:8px;
}

/* for IDs see the class: org.eclipse.e4.demo.cssbridge.ui.views.Theme.FolderPreviewView */
FontDefinition#org-eclipse-e4-demo-cssbridge-ui-views-theme-folderpreviewview-low-imp-mail-font {
	font-family: Terminal;
	font-size: 8px;
}

FontDefinition#org-eclipse-e4-demo-cssbridge-ui-views-theme-folderpreviewview-normal-imp-mail-font {
	font-family: Tahoma;
	font-size: 8px;
}

FontDefinition#org-eclipse-e4-demo-cssbridge-ui-views-theme-folderpreviewview-high-imp-mail-font {
	font-family: Times;
	font-size: 11px;
	font-style: italic;
}

Shell, Tree, Table, Text {
	background-color: '#org-eclipse-e4-demo-cssbridge-ui-views-theme-shell-background';
	font-family: '#org-eclipse-ui-workbench-TAB_TEXT_FONT';
}
