/*******************************************************************************
 * Copyright (c) 2010, 2019 Andrea Guarinoni and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     Andrea Guarinoni <andrea.guarinoni.dev@outlook.com> - initial API and implementation
 *******************************************************************************/

@import url("platform:/plugin/org.eclipse.ui.themes/css/common/e4_globalstyle.css");
@import url("platform:/plugin/org.eclipse.ui.themes/css/dark/e4-dark_ide_colorextensions.css");
@import url("platform:/plugin/org.eclipse.ui.themes/css/dark/e4-dark_globalstyle.css"); /* Remove this to have ONLY the main IDE shell dark */
@import url("platform:/plugin/org.eclipse.ui.themes/css/dark/e4-dark_partstyle.css");
@import url("platform:/plugin/org.eclipse.ui.themes/css/dark/e4-dark_tabstyle.css");
@import url("platform:/plugin/org.eclipse.ui.themes/css/dark/e4-dark-drag-styling.css");

.MToolControl.TrimStack {
    /*frame-image:  url(./gtkTSFrame.png);*/
    handle-image:  url(./dragHandle.png);
    /*frame-image-rotated:  url(./gtkTSFrame-rotated.png);*/
    handle-image-rotated:  url(./dragHandle-rotated.png);
    frame-cuts: 5px 1px 5px 16px;
}

.MPartStack.active CTabFolder Canvas {
    background-color: #262626;
    color: #CCC;
}


/* #################### Bottom Status Bar ######################## */

StatusLine,
ImageBasedFrame,
#org-eclipse-ui-StatusLine,
#org-eclipse-ui-StatusLine CLabel,
#org-eclipse-ui-ProgressBar,
#org-eclipse-ui-ProgressBar Canvas {
     color:'#org-eclipse-ui-workbench-DARK_FOREGROUND'; 
}

/* ###################### Global Styles ########################## */

/* Use unset to set the foreground/background color to null */

Table[swt-lines-visible=true] {
    background-color: unset;
}

Tree[swt-lines-visible=true] {
    background-color: unset;
}

Button  {
    background-color: unset;
    color: unset;
}

